import json
import os


def read_json_to_dictionary(path):
    file = os.getcwd() + r'\iso8583_dlib\segments' + path
    try:
        f = open(file, 'r')
        data = json.load(f)
        f.close()
        return data
    except NameError:
        print("Error: ", NameError)
        return {}



"""Main Parser Class"""

import json
from .segments.utilities.operations import convert_bitmap_to_active_bits
from .segments.iso import ISO
from .segments.header import Header
from .segments.mti import MTI
from .segments.primary_bitmap import PrimaryBitmap
from .segments.data_elements import DataElements


class Parser:
    """Main parser class"""

    def __init__(self, message):
        """Init class require a message as input"""
        # Parser variables
        self.message = message
        self.active_data_elements = []
        self.json_data = None

        # Parser segments
        self.iso = self.message[ISO.start_position:ISO.end_position]
        self.header = self.message[Header.start_position:Header.end_position]
        self.mti = self.message[MTI.start_position:MTI.end_position]
        self.primary_bitmap = self.message[PrimaryBitmap.start_position:PrimaryBitmap.end_position]
        self.data_elements = self.message[DataElements.start_position:]

        # Parser function
        self.iso_8583()

    def iso_8583(self):
        """Processing data"""

        self.active_data_elements = convert_bitmap_to_active_bits(self.primary_bitmap)

    def get_json(self, save=False):
        """Return json format data"""
        print(self.active_data_elements)
        to_convert = {
            "literal": self.iso,
            "header": Header.get_message(self.header),
            "mti": MTI.get_type(self.mti),
            "primary_bitmap": self.primary_bitmap,
            "data_elements": DataElements.get_all_data_elements(self.active_data_elements, self.data_elements)
        }
        if save:
            with open('data.json', 'w', encoding='utf-8') as f:
                json.dump(to_convert, f, ensure_ascii=False, indent=4)

        return json.dumps(to_convert)



"""Third segment"""
from .read_metadata_segments import read_json_to_dictionary


class MTI:
    start_position = 12
    end_position = 16

    @staticmethod
    def get_type(select):
        return read_json_to_dictionary(r'\data\mti\mti_types.json')[select]



"""Fifth segment"""
from .read_metadata_segments import read_json_to_dictionary
from .utilities.operations import convert_bitmap_to_active_bits


class DataElements:
    start_position = 32

    @staticmethod
    def get_element(select):
        return read_json_to_dictionary(r'\data\data_elements\list_elements.json')[select]

    @staticmethod
    def get_all_data_elements(active_elements, data_elements):
        data = [{"Active_elements": active_elements}]
        for i in active_elements:
            # Only support the first seven data elements
            if i <= 14:
                element = DataElements.process_element(DataElements.get_element(str(i)), data_elements)
                data.append(element)
                pass
        return data

    @staticmethod
    def process_element(element, data_elements):
        result = {}
        data = {}
        value = data_elements[element["start_position"]:element["end_position"]]
        data["value"] = value
        if element["bitmap"]:
            data["active_elements"] = convert_bitmap_to_active_bits(bitmap=value,
                                                                    position=65)
        if element["dynamic"]:
            # For implement
            pass
        else:
            pass
        data["data_element"] = element["element"]
        data["description"] = element["description"]
        data["len"] = element["len"]
        data["format"] = element["format"]
        result[str(element["element"])] = data
        return result



import json
import os


def hex_to_binary(string):
    n = int(string, 16)
    b_str = ''
    while n > 0:
        b_str = str(n % 2) + b_str
        n = n >> 1
    return b_str.zfill(4)


def convert_bitmap_to_active_bits(bitmap, position=1):
    active_bits = []
    for element in bitmap:
        for bit in hex_to_binary(element):
            if bit == '1':
                active_bits.append(position)
            position += 1
    return active_bits



"""Second segment"""

from .read_metadata_segments import read_json_to_dictionary


class Header:
    start_position = 3
    end_position = 12

    @staticmethod
    def get_header_pi(select):
        return read_json_to_dictionary(r'\data\header\header_PI.json')[select]

    @staticmethod
    def get_header_rn(select):
        return read_json_to_dictionary(r'\data\header\header_RN.json')[select]

    @staticmethod
    def get_header_status(select):
        return read_json_to_dictionary(r'\data\header\header_Status.json')[select]

    @staticmethod
    def get_header_oc(select):
        return read_json_to_dictionary(r'\data\header\header_OC.json')[select]

    @staticmethod
    def get_header_rc(select):
        return read_json_to_dictionary(r'\data\header\header_RC.json')[select]

    @staticmethod
    def get_message(header):
        return {
            "Complete_header": header,
            "Product_indicator": Header.get_header_pi(header[0:2]),
            "Release_number": Header.get_header_rn(header[2:4]),
            "Status": Header.get_header_status(header[4:7]),
            "Originator_code": Header.get_header_oc(header[7:8]),
            "Responder_code": Header.get_header_rc(header[8:9]),
        }



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

project_urls = {
    "Homepage": "https://github.com/eocode/ISO8583-Dlib",
    "Issue tracker": "https://github.com/eocode/ISO8583-Dlib/issues",
    "Code": "https://github.com/eocode/ISO8583-Dlib",
    "Documentation": "https://github.com/eocode/ISO8583-Dlib/wiki",
}

setuptools.setup(
    name='ISO8583-Dlib',
    version='0.1.3',
    author='eocode',
    author_email='hola@eliasojedamedina.com',
    license='MIT license',
    description='Parser String to JSON for ISO8583 Package',
    long_description_content_type="text/markdown",
    long_description=open("README.md").read()
    + "\n\n"
    + open("CHANGELOG.md").read(),
    packages=setuptools.find_packages(exclude=("tests",)),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    project_urls=project_urls,
    install_requires=[],
    python_requires=">=3.6",
)
